[PATCH 4/4] aic94xx: nmi timeout fix

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

 



Few of us have seen the AIC94XX driver get stuck reading the
ATOMICSTATCTL register in a loop for ever. The possible immortal loop
in asd_ddbsite_update_word() is made mortal now! Retry worked just
fine!

Signed-off-by: Malahal Naineni <malahal@xxxxxxxxxx>

---

diff -r 5f7a0e8a8265 drivers/scsi/aic94xx/aic94xx_reg.h
--- a/drivers/scsi/aic94xx/aic94xx_reg.h	Wed Oct 25 13:28:35 2006 -0700
+++ b/drivers/scsi/aic94xx/aic94xx_reg.h	Fri Oct 27 10:22:59 2006 -0700
@@ -226,20 +226,30 @@ static inline int asd_ddbsite_update_wor
 					  u16 oldval, u16 newval)
 {
 	u8  done;
+	int retries = 100;
+	int i;
+
 	u16 oval = asd_ddbsite_read_word(asd_ha, ddb_site_no, offs);
 	if (oval != oldval)
 		return -EAGAIN;
 	asd_write_reg_word(asd_ha, AOLDDATA, oldval);
 	asd_write_reg_word(asd_ha, ANEWDATA, newval);
-	do {
+	for (i = 0; i < retries; i++) {
 		done = asd_read_reg_byte(asd_ha, ATOMICSTATCTL);
-	} while (!(done & ATOMICDONE));
-	if (done & ATOMICERR)
-		return -EFAULT;	  /* parity error */
-	else if (done & ATOMICWIN)
-		return 0;	  /* success */
-	else
-		return -EAGAIN;	  /* oldval different than current value */
+		if (done & ATOMICDONE) {
+			if (done & ATOMICERR)
+				return -EFAULT;	  /* parity error */
+			else if (done & ATOMICWIN)
+				return 0;	  /* success */
+			else
+				/* oldval different than current value */
+				return -EAGAIN;
+		}
+	}
+
+	/* ATOMICDONE is not set after repeated retries! Broken chip? */
+	WARN_ON(1);
+	return -EAGAIN;
 }
 
 static inline int asd_ddbsite_update_byte(struct asd_ha_struct *asd_ha,

-
To unsubscribe from this list: send the line "unsubscribe linux-scsi" in
the body of a message to majordomo@xxxxxxxxxxxxxxx
More majordomo info at  http://vger.kernel.org/majordomo-info.html

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]
[Index of Archives]     [SCSI Target Devel]     [Linux SCSI Target Infrastructure]     [Kernel Newbies]     [IDE]     [Security]     [Git]     [Netfilter]     [Bugtraq]     [Yosemite News]     [MIPS Linux]     [ARM Linux]     [Linux Security]     [Linux RAID]     [Linux ATA RAID]     [Linux IIO]     [Samba]     [Device Mapper]
  Powered by Linux